// import 'source-map-support/register';
import { RegisterSchema, SchemaProperty } from 'koishi-thirdeye';
import { Adapter, segment } from 'koishi';
import _ from 'lodash';
import { AlertDto } from './def';
import { SendTarget } from 'koishi-target-def';

@RegisterSchema()
export class Instance {
  @SchemaProperty({
    required: true,
    description: 'POST 的 Authorization Bearer 字段，用于标识不同路由。',
  })
  token: string;

  @SchemaProperty({
    description: '警报标题。',
    default: '警报',
  })
  title: string;

  @SchemaProperty({
    description: '解除警报的提示信息。',
    default: '警报已解除。',
  })
  resolvedMessage: string;

  @SchemaProperty({
    description: '使用 markdown 语法。',
    default: false,
  })
  markdown: boolean;

  formatAlert(dto: AlertDto) {
    const { alerts } = dto;
    let text = `${this.markdown ? segment('markdown') : ''}# ${this.title}`;

    for (const alert of alerts) {
      text += `\n\n## ${
        alert.annotations.summary || alert.labels.alertname
      }\n\n`;
      for (const [key, value] of Object.entries(alert.labels)) {
        // if (key === 'alertname') continue;
        text += `* ${key}: \`${value}\`\n`;
      }
      text += `\n`;
      if (alert.status === 'resolved') {
        text += this.resolvedMessage;
        continue;
      }
      text += `${alert.annotations.description}`;
    }
    return text;
  }

  @SchemaProperty({
    type: SendTarget,
    description: '消息发送目标。',
    default: [],
  })
  targets: SendTarget[];

  async send(bots: Adapter.BotList, content: string) {
    return _.flatten(
      await Promise.all(
        this.targets.map((target) => target.send(bots, content)),
      ),
    );
  }
}

@RegisterSchema()
export class AlertmanagerPluginConfig {
  constructor(config: AlertmanagerPluginConfigLike) {}

  @SchemaProperty({
    description: '请求路径。',
    default: '/alertmanager',
  })
  path: string;

  @SchemaProperty({ type: Instance })
  instances: Instance[];

  getInstance(header: string) {
    const token = header?.startsWith('Bearer ') ? header.slice(7) : header;
    return this.instances.find((instance) => instance.token === token);
  }
}

export type AlertmanagerPluginConfigLike = Partial<AlertmanagerPluginConfig>;
